import numpy as np
import pandas as pd
import wrds

END_DATE = pd.to_datetime('12-31-2016')


def download(save=True):
    """Download Fama-Bliss data from WRDS. If save is True, save output to a csv. Else, return it."""
    with wrds.Connection() as conn:
        df = conn.raw_sql("SELECT kytreasnox, mcaldt, tmnomprc FROM crsp.tfz_mth_fb WHERE mcaldt <= DATE '2018-12-31'",
                          date_cols=['mcaldt'])

    BOND_CODES = {2000047: 'USD1Y', 2000048: 'USD2Y', 2000049: 'USD3Y', 2000050: 'USD4Y', 2000051: 'USD5Y'}
    assert df['kytreasnox'].isin(list(BOND_CODES.keys())).all(), \
        'Fama-Bliss dataset should only contain observations for 1-5 year bond yields. If not, something has changed.'

    df = df.pivot(index='mcaldt', columns='kytreasnox', values='tmnomprc').reset_index()
    df = df.rename(columns=BOND_CODES).rename(columns={'mcaldt': 'date'})
    df.index.name = None
    if save:
        df.to_csv('raw_data/Fama_Bliss_data.csv', index=False)
        pass
    else:
        return df


def create_us(save=True):
    try:
        df = pd.read_csv('raw_data/Fama_Bliss_data.csv', parse_dates=['date'])
    except FileNotFoundError:
        df = download(False)

    START_DATE = pd.to_datetime('06-30-1952')

    df = df[(df['date'] >= START_DATE) & (df['date'] <= END_DATE)]

    df['date'] = df['date'] + pd.tseries.offsets.MonthEnd(0)

    # Convert prices to yields
    df[['USD1Y', 'USD2Y', 'USD3Y', 'USD4Y', 'USD5Y']] = \
        -np.log(df[['USD1Y', 'USD2Y', 'USD3Y', 'USD4Y', 'USD5Y']] / 100).multiply([1, 1/2, 1/3, 1/4, 1/5]) * 100
    
    if save:
        df.to_csv('USD.csv', index=False)
        pass
    else:
        return df
