"""Main script to run analyses for the project."""
import sys
import time

import seaborn as sns
from matplotlib import rcParams

from modules.tables_and_figures import table3
from modules.tables_and_figures import table4
from modules.tables_and_figures import table5
from modules.tables_and_figures import table6_figures1_2
from modules.tables_and_figures import table7
from modules.tables_and_figures import tableA3
from modules.tables_and_figures import tableA4
from modules.tables_and_figures import tables1_2
from modules.tables_and_figures import tablesA1_A2


def main(output='console'):
    """Main entry point for the analysis."""
    t = time.time()
    assert output in ['console', 'file'], 'First argument must be console or file.'

    # Graph settings
    rcParams.update({'figure.autolayout': True})
    sns.set_style('ticks')
    sns.set_context('notebook')
    if output == 'file':
        sns.set_palette('Greys_r')
    else:
        sns.set_palette('deep')

    # Run analyses
    tables1_2.create(output=output)
    table3.create(output=output)
    table4.create(output=output, base_country='USD')
    table5.create(output=output, base_country='USD')
    table6_figures1_2.create(output=output)
    table7.create(output=output, base_country='USD')

    tablesA1_A2.create(output=output)
    tableA3.create(output=output)
    tableA4.create(output=output)

    print('\nExecution finished in %s seconds.' % round(time.time() - t, 2))
    pass


if __name__ == '__main__':
    if len(sys.argv) > 1:
        output = sys.argv[1]
    else:
        output = 'console'
    sys.exit(main(output=output))
