import numpy as np
import statsmodels.formula.api as sm
from sklearn.decomposition import PCA


def run(df, y_variables, x_variables, n_components=1):
    """
    Run unrestricted regression on each excess return using all forward spreads, and calculate the percentage of the
    variance of the fitted values explained by their first n principal components.

    Parameters
    ----------
    df: pandas.DataFrame
        A dataframe that contains time series for all excess bond returns and forward spreads (for one country).
    y_variables: list
        A list of df column names that contain excess bond returns used in the analysis.
    y_variables: list
        A list of df column names that contain forward spreads used in the analysis.
    n_components: integer, default 1
        Number of components used to explain variance.

    Output
    ------
    Float
    """
    fittedvalues = []
    for n in y_variables:
        ols = sm.ols('{} ~ {}'.format(n, ' + '.join(x_variables)), data=df).fit()
        fittedvalues += [ols.fittedvalues.tolist()]

    return np.asscalar(PCA(n_components=n_components).fit(np.array(fittedvalues).T).explained_variance_ratio_)
