import matplotlib.pyplot as plt
import pandas as pd

from modules import load_data
from modules.analyses import cp2stage


def create(output='console', country='USD'):
    OOS_START = '1/1/2004'

    Y_VARIABLES = ['rx2_fwd', 'rx3_fwd', 'rx4_fwd', 'rx5_fwd']
    X_VARIABLES = ['fs2_bar', 'fs3_bar', 'fs4_bar', 'fs5_bar']

    df = load_data.load()

    t_gamma = []
    t_b = []
    for t in df.loc[df['date'] >= OOS_START, 'date'].unique():
        df_temp = df[(df['country'] == country) & (df['date'] <= t)].copy()
        cp_result = cp2stage.run(df_temp, Y_VARIABLES, X_VARIABLES)

        t_gamma += [[t] + cp_result['gammas']['params'].tolist()]
        t_b += [[t] + cp_result['bs']['params'].tolist()]

    cols = ['date'] + ['k={}'.format(i) for i in range(1, len(t_gamma[0]))]
    dfg = pd.DataFrame(t_gamma, columns=cols).set_index('date')

    cols = ['date'] + ['k={}'.format(i) for i in range(1, len(t_b[0]))]
    dfb = pd.DataFrame(t_b, columns=cols).set_index('date')

    fig, ax = plt.subplots(nrows=2, ncols=1, figsize=(10, 7))

    dfg.plot(ax=ax[0])
    ax[0].set_xlabel('')
    ax[0].set_ylabel('$\\gamma_k$')
    ax[0].set_xlim([OOS_START, dfb.index.max().strftime('%m/%d/%Y')])
    ax[0].legend(loc='lower right')

    dfb.plot(ax=ax[1])
    ax[1].set_xlabel('')
    ax[1].set_ylabel('$b_k$')
    ax[1].set_xlim([OOS_START, dfb.index.max().strftime('%m/%d/%Y')])
    ax[1].set_ylim((0, dfb.max().max()*1.2))
    ax[1].legend(loc='lower right')

    if output == 'file':
        plt.savefig('output/figures/T1OosParams{}.eps'.format(country), format='eps')

    pass
