import numpy as np
import pandas as pd
from prettytable import PrettyTable
from sklearn.decomposition import PCA

from modules import load_data
from modules.analyses import cp2stage
from modules.utils.table import Table


def create(output='console'):
    SUBSAMPLES = [['12/31/1973', '12/31/2002'],
                  ['12/31/2003', '12/31/2015']]

    Y_VARIABLES = ['rx2_fwd', 'rx3_fwd', 'rx4_fwd', 'rx5_fwd']
    X_VARIABLES = ['fs2_bar', 'fs3_bar', 'fs4_bar', 'fs5_bar']

    df = load_data.load()

    tdata = {}
    for n, j in enumerate(SUBSAMPLES):
        df2 = pd.DataFrame()
        for i in df['country'].unique():
            df_temp = df[(df['country'] == i) & (df['date'] >= j[0]) & (df['date'] <= j[1])].copy()

            df_temp = cp2stage.add_cp_factor_and_resid(df_temp, Y_VARIABLES, X_VARIABLES)

            df2 = df2.append(df_temp[['country', 'date', 'cp_factor']])

        countries = list(df2['country'].unique())
        df2 = df2.pivot(index='date', columns='country', values='cp_factor')[countries]

        df3 = df2.corr()
        df2 = df2.dropna(how='any')
        df3[~np.triu(df3.values).astype(np.bool)] = ''
        df3['i'] = range(1, len(countries) + 1)
        df3['PC(i)'] = PCA(n_components=len(countries)).fit(df2[countries].values).explained_variance_ratio_

        df3.insert(0, '', countries)
        tdata[n] = df3

    print('\nTable 3')
    if output == 'console':

        for n, td in tdata.items():
            table = PrettyTable(list(td.columns))
            table.float_format = '0.2'
            for i in td.values:
                table.add_row(i)
            print(table)

    elif output == 'file':
        panel_names = ['Panel A: Pre-2004 Data',
                       'Panel B: Post-2003 Data']
        colnames = list(tdata[0].columns[:-1]) + ['\\%PC(i)']
        justs = ['l'] + ['r'] * (len(colnames) - 1)

        table = Table(len(colnames), justs=justs, label='CPcorrPre_Post',
                      caption='Correlation Matrices and Variance Decompositions of Country CP Factors',
                      caption_position='bottom')

        for n, td in tdata.items():
            table.add_panel(td,
                            supheaders=[panel_names[n]],
                            supheader_cols=[(1, len(colnames))],
                            headers=colnames,
                            formatting='%.2f')

        table.print_table(fp='output/tables/CPcorrPre_Post.tex')

    pass
