import pandas as pd
from prettytable import PrettyTable

from modules import load_data
from modules.analyses import curr2stage
from modules.utils.table import Table


def create(output='console', base_country='USD'):
    Y_VARIABLES = ['rx2_fwd', 'rx3_fwd', 'rx4_fwd', 'rx5_fwd']
    X_VARIABLES = ['fs2_bar', 'fs3_bar', 'fs4_bar', 'fs5_bar']

    df = load_data.load()

    tdata = []
    for i in df['country'].unique():
        if i == base_country:
            continue

        curr_result = curr2stage.run(df, Y_VARIABLES, X_VARIABLES, 't4', [i, base_country])

        tdata += [[i] + curr_result['params']['params'].tolist() + [curr_result['rsquared_adj']]]
        tdata += [[''] + curr_result['params']['se'].tolist() + ['']]

    print('\nTable 4')
    if output == 'console':
        table = PrettyTable(['CUR j', 'beta0', 'beta1', 'beta2', 'beta3', 'beta4', 'R-squared'])
        table.float_format = '0.2'
        for i in tdata:
            table.add_row(i)
        print(table)

    elif output == 'file':
        colnames = ['CUR $j$', '$\\beta_0$', '$\\beta_1$', '$\\beta_2$', '$\\beta_3$', '$\\beta_4$', '$R^2$']

        justs = ['l'] + ['d'] * (len(colnames) - 1)

        table = Table(len(colnames), justs=justs, label='Level_pred{}'.format(base_country),
                      caption='Explaining Currency Market Excess Returns with Changes in Level Factors',
                      caption_position='bottom')

        table.add_panel(pd.DataFrame(tdata), headers=colnames, formatting='%.2f',
                        regression_specs={'stars': None, 'dof': None, 'means': None, 'bracket_marker': '('})

        table.print_table(fp='output/tables/Level_pred{}.tex'.format(base_country))
