import pandas as pd
from prettytable import PrettyTable

from modules import load_data
from modules.analyses import curr2stage
from modules.utils.table import Table


def create(output='console', base_country='USD'):
    SUBSAMPLES = [['12/31/1973', '12/31/2002'],
                  ['12/31/2003', '12/31/2015']]

    Y_VARIABLES = ['rx2_fwd', 'rx3_fwd', 'rx4_fwd', 'rx5_fwd']
    X_VARIABLES = ['fs2_bar', 'fs3_bar', 'fs4_bar', 'fs5_bar']

    df = load_data.load()

    tdata = []
    for i in df['country'].unique():
        if i == base_country:
            continue
        for n, j in enumerate(SUBSAMPLES):
            df_temp = df[(df['date'] >= j[0]) & (df['date'] <= j[1])].copy()
            curr_result = curr2stage.run(df_temp, Y_VARIABLES, X_VARIABLES, 't5', [i, base_country])

            tdata += [[i + str(n+1)] + curr_result['params']['params'].tolist()
                      + [curr_result['wald_test_cp']['value'], curr_result['wald_test_uip0']['value'],
                         curr_result['wald_test_uip1']['value'], curr_result['rsquared_adj']]]
            tdata += [[''] + curr_result['params']['se'].tolist()
                      + [curr_result['wald_test_cp']['p_value'], curr_result['wald_test_uip0']['p_value'],
                         curr_result['wald_test_uip1']['p_value'], '']]
    print('\nTable 5')
    if output == 'console':
        table = PrettyTable(['CUR j', 'phi0', 'phi1', 'phi2', 'phi3', 'chi2_cp', 'chi2_uip0',
                             'chi2_uip1', 'R-squared'])
        table.float_format = '0.2'
        for i in tdata:
            table.add_row(i)
        print(table)

    elif output == 'file':
        colnames = ['CUR $j$', '$\\phi_0$', '$\\phi_1$', '$\\phi_2$', '$\\phi_3$',
                    '$\\chi^2 ({})_{{\\phi_1,\\phi_2=0}}$'.format(curr_result['wald_test_cp']['dof']),
                    '$\\chi^2 ({})_{{\\phi_3=0}}$'.format(curr_result['wald_test_uip0']['dof']),
                    '$\\chi^2 ({})_{{\\phi_3=1}}$'.format(curr_result['wald_test_uip1']['dof']), '$R^2$']

        justs = ['l'] + ['d'] * (len(colnames) - 1)
        brackets = ['('] * 5 + ['angled'] * 4

        table = Table(len(colnames), justs=justs, label='CurrencyRetForecasts{}'.format(base_country),
                      caption='Forecasts of {} Excess Rates of Return in Currency Markets'.format(base_country),
                      caption_position='bottom')

        for row1, row2, row3, row4 in zip(tdata[0::4], tdata[1::4], tdata[2::4], tdata[3::4]):
            table.add_panel(pd.DataFrame([row1, row2, row3, row4]), headers=colnames,
                            formatting='%.2f',
                            regression_specs={'stars': None, 'dof': None, 'means': None, 'bracket_marker': brackets})
            colnames = None

        table.print_table(fp='output/tables/CurrencyRetForecasts{}.tex'.format(base_country))
