import pandas as pd
from prettytable import PrettyTable

from modules import load_data
from modules.analyses import oos_curr
from modules.utils.table import Table


def create(output='console', base_country='USD'):
    OOS_START = '12/31/2003'

    Y_VARIABLES = ['rx2_fwd', 'rx3_fwd', 'rx4_fwd', 'rx5_fwd']
    X_VARIABLES = ['fs2_bar', 'fs3_bar', 'fs4_bar', 'fs5_bar']

    df = load_data.load()

    tdata = []
    for i in df['country'].unique():
        if i == base_country:
            continue
        oos_result = oos_curr.run(df, Y_VARIABLES, X_VARIABLES, OOS_START, [i, base_country])

        tdata += [[i, oos_result['rsquared'], oos_result['mse_f']]]

    print('\nTable 7')
    if output == 'console':
        table = PrettyTable(['CUR', 'R-squared', 'MSE-F'])
        table.float_format = '0.2'
        for i in tdata:
            table.add_row(i)
        print(table)

    elif output == 'file':
        colnames = ['CUR', '$R^2$', 'MSE-F']

        justs = ['l'] + ['r'] * (len(colnames) - 1)
        caption = 'Out-of-Sample Forecasts for Excess Foreign Exchange Returns: Cochrane-Piazzesi Factors vs. Historical Means'

        table = Table(len(colnames), justs=justs, label='OoS_forecasts_FX_{}'.format(base_country),
                      caption=caption, caption_position='bottom')

        table.add_panel(pd.DataFrame(tdata),
                        headers=colnames,
                        formatting='%.2f')

        table.print_table(fp='output/tables/OoS_forecasts_FX_{}.tex'.format(base_country))

    pass
