import pandas as pd
from prettytable import PrettyTable

from modules import load_data
from modules.analyses import oos_constrained
from modules.utils.table import Table


def create(output='console'):
    START_DATE = '3/31/1988'
    OOS_START = '12/31/2003'

    Y_VARIABLES = ['rx2_fwd', 'rx3_fwd', 'rx4_fwd', 'rx5_fwd']
    X_VARIABLES = ['fs2_bar', 'fs3_bar', 'fs4_bar', 'fs5_bar']

    df = load_data.load()
    df = df[(df['date'] >= START_DATE) & (df['country'] != 'NOK')].copy()

    oos_result = oos_constrained.run(df, Y_VARIABLES, X_VARIABLES, OOS_START)

    tdata = []
    for i in df['country'].unique():
        tdata += [[i] + list(oos_result[i]['rsquared']) + list(oos_result[i]['mse_f'])]

    print('\nTable A3')
    if output == 'console':
        table = PrettyTable(['CUR', 'R2(rx2)', 'R2(rx3)', 'R2(rx4)', 'R2(rx5)',
                             'MSE-F(rx2)', 'MSE-F(rx3)', 'MSE-F(rx4)', 'MSE-F(rx5)'])
        table.float_format = '0.2'
        for i in tdata:
            table.add_row(i)
        print(table)

    elif output == 'file':
        colnames = ['CUR'] + ['$rx_{{t+1}}^{{({})}}$'.format(i) for i in range(2, 6)] * 2
        justs = ['l'] + ['r'] * (len(colnames) - 1)

        caption = 'Out-of-Sample Forecasts of Excess Bond Returns: Restricted Cochrane-Piazzesi Models vs. Historical Means'

        table = Table(len(colnames), justs=justs, label='OoS_forecasts_CPrestricted',
                      caption=caption,
                      caption_position='bottom')

        table.add_panel(pd.DataFrame(tdata),
                        supheaders=['', '$R^2$', 'MSE-F'],
                        supheader_cols=[1, (2, 5), (6, 9)],
                        headers=colnames,
                        formatting='%.2f')

        table.print_table(fp='output/tables/OoS_forecasts_CPrestricted.tex')

    pass
