import pandas as pd
from prettytable import PrettyTable

from modules import load_data
from modules.analyses import curr2stage
from modules.utils.table import Table


def create(output='console'):
    T1_BASE = ['GBP']
    T2_BASE = ['EUR', 'CHF']
    T3_BASE = ['CAD', 'JPY', 'AUD', 'SEK']
    SUBSAMPLES = [['12/31/1973', '12/31/2002'],
                  ['12/31/2003', '12/31/2015']]

    Y_VARIABLES = ['rx2_fwd', 'rx3_fwd', 'rx4_fwd', 'rx5_fwd']
    X_VARIABLES = ['fs2_bar', 'fs3_bar', 'fs4_bar', 'fs5_bar']

    df = load_data.load()
    df = df[df['country'] != 'USD']

    for tablenum, baselist in enumerate([T1_BASE, T2_BASE, T3_BASE]):
        tdata_dict = {}
        curr_result_dict = {}
        for i in baselist:
            tdata_dict[i] = []
            for j in df['country'].unique():
                if j == i:
                    continue
                for n, k in enumerate(SUBSAMPLES):
                    df_temp = df[(df['date'] >= k[0]) & (df['date'] <= k[1])].copy()
                    curr_result = curr2stage.run(df_temp, Y_VARIABLES, X_VARIABLES, 't5', [j, i])
                    curr_result_dict[i] = curr_result

                    tdata_dict[i] += [[j + str(n+1)] + curr_result['params']['params'].tolist()
                                   + [curr_result['wald_test_cp']['value'], curr_result['wald_test_uip0']['value'],
                                      curr_result['wald_test_uip1']['value'], curr_result['rsquared_adj']]]
                    tdata_dict[i] += [[''] + curr_result['params']['se'].tolist()
                                   + [curr_result['wald_test_cp']['p_value'], curr_result['wald_test_uip0']['p_value'],
                                      curr_result['wald_test_uip1']['p_value'], '']]
            df = df[df['country'] != i]
        _print_table(output, tdata_dict, curr_result_dict, tablenum+1)



def _print_table(output, tdata_dict, curr_result_dict, tablenum):
    print('\nTable A4 ({})'.format(tablenum))
    if output == 'console':
        for i in tdata_dict.keys():
            print('Base Currency = {}'.format(i))
            table = PrettyTable(['CUR j', 'phi0', 'phi1', 'phi2', 'phi3', 'chi2_cp', 'chi2_uip0',
                                 'chi2_uip1', 'R-squared'])
            table.float_format = '0.2'
            for j in tdata_dict[i]:
                table.add_row(j)
            print(table)

    elif output == 'file':
        c = list(tdata_dict.keys())[0]
        supheaders = ['CUR $j$', '$\\phi_0$', '$\\phi_1$', '$\\phi_2$', '$\\phi_3$',
                      '$\\chi^2 ({})_{{\\phi_1,\\phi_2=0}}$'.format(curr_result_dict[c]['wald_test_cp']['dof']),
                      '$\\chi^2 ({})_{{\\phi_3=0}}$'.format(curr_result_dict[c]['wald_test_uip0']['dof']),
                      '$\\chi^2 ({})_{{\\phi_3=1}}$'.format(curr_result_dict[c]['wald_test_uip1']['dof']), '$R^2$']

        justs = ['l'] + ['d'] * (len(supheaders) - 1)
        brackets = ['('] * 5 + ['angled'] * 4

        table = Table(len(supheaders), justs=justs, label='CurrencyRetForecastsExtra{}'.format(tablenum),
                      caption='Forecasts of Excess Rates of Return in Currency Markets',
                      caption_position='bottom')
        for n, i in enumerate(tdata_dict.keys()):
            colname = ['Base Currency = {}'.format(i)]
            for row1, row2, row3, row4 in zip(tdata_dict[i][0::4], tdata_dict[i][1::4],
                                              tdata_dict[i][2::4], tdata_dict[i][3::4]):

                table.add_panel(pd.DataFrame([row1, row2, row3, row4]), supheaders=supheaders,
                                headers=colname, header_cols=[(1, 9)],
                                formatting='%.2f',
                                regression_specs={'stars': None, 'dof': None, 'means': None, 'bracket_marker': brackets})
                supheaders = None
                colname = None

        table.print_table(fp='output/tables/CurrencyRetForecastsExtra{}.tex'.format(tablenum))
