import pandas as pd
from prettytable import PrettyTable

from modules import load_data
from modules.analyses import cp_freeconstant
from modules.utils.table import Table


def create(output='console'):
    SUBSAMPLES = [['12/31/1973', '12/31/2002'],
                  ['12/31/2003', '12/31/2015']]

    Y_VARIABLES = ['rx2_fwd', 'rx3_fwd', 'rx4_fwd', 'rx5_fwd']
    X_VARIABLES = ['fs2_bar', 'fs3_bar', 'fs4_bar', 'fs5_bar']

    df = load_data.load()

    tdata = {}
    tdata2 = {}
    for n, j in enumerate(SUBSAMPLES):
        tdata[n] = []
        tdata2[n] = []
        for i in df['country'].unique():
            df_temp = df[(df['country'] == i) & (df['date'] >= j[0]) & (df['date'] <= j[1])].copy()

            cp_result = cp_freeconstant.run(df_temp, Y_VARIABLES, X_VARIABLES)

            tdata[n] += [[i + str(n+1)]
                         + cp_result['alphas']['params'].tolist()
                         + cp_result['deltas']['params'].tolist() + ['']]

            tdata[n] += [['']
                         + cp_result['alphas']['se'].tolist()
                         + cp_result['deltas']['se'].tolist() + ['']]

            tdata2[n] += [[i + str(n+1)]
                          + cp_result['bs']['params'].tolist()[:-1]
                          + cp_result['gammas']['params'].tolist()
                          + [cp_result['J_test']['value'], cp_result['rsquared']]]

            tdata2[n] += [['']
                          + cp_result['bs']['se'].tolist()[:-1]
                          + cp_result['gammas']['se'].tolist()
                          + [cp_result['J_test']['p_value'], '']]

    print('\nTable A1 & A2'.format(n+1))
    if output == 'console':
        for n in tdata.keys():
            table = PrettyTable(['CUR', 'alpha2', 'alpha3', 'alpha4', 'alpha5',
                                 'delta2', 'delta3', 'delta4', 'delta5', ''])
            table.float_format = '0.2'
            for i in tdata[n]:
                table.add_row(i)
            print(table)

            table2 = PrettyTable(['CUR', 'b2', 'b3', 'b4', 'gamma2', 'gamma3', 'gamma4', 'gamma5',
                                  'J-stat', 'R-squared'])
            table2.float_format = '0.2'
            for i in tdata2[n]:
                table2.add_row(i)
            print(table2)

    elif output == 'file':
        for n in tdata.keys():
            suffix = 'Pre' if n == 0 else 'Post'
            title_suffix = '(Pre-2004 Data)' if n == 0 else '(Post-2003 Data)'
            colnames = ['CUR'] + ['$\\alpha_{}$'.format(i) for i in range(2, 6)] \
                + ['$\\delta_{}$'.format(i) for i in range(2, 6)] + ['']

            colnames2 = ['CUR'] + ['$b_{}$'.format(i) for i in range(2, 5)] \
                + ['$\\gamma_{}$'.format(i) for i in range(2, 6)] + ['J-stat', '$R^2$']

            brackets1 = '('
            brackets2 = ['('] * 8 + ['angled'] * 2
            justs = ['l'] + ['d'] * (len(colnames) - 1)

            table = Table(len(colnames), justs=justs, label='SingleFactorModelFreeConst{}'.format(suffix),
                          caption='The Single Factor Model with Free Constants '.format(title_suffix),
                          caption_position='bottom')
            table.add_panel(pd.DataFrame(tdata[n]),
                            headers=colnames,
                            formatting='%.2f',
                            regression_specs={'stars': None, 'dof': None, 'means': None, 'bracket_marker': brackets1})
            table.add_panel(pd.DataFrame(tdata2[n]),
                            headers=colnames2,
                            formatting='%.2f',
                            regression_specs={'stars': None, 'dof': None, 'means': None, 'bracket_marker': brackets2})
            table.print_table(fp='output/tables/SingleFactorModelFreeConst{}.tex'.format(suffix))
    pass
