def S_matrix(u, lags, newey_west=False, demean=True):
    """
    Calculate spectral density matrix. Python implementation of John Cochrane's spectralmatrix.m.

    Parameters
    ----------
    u: numpy.matrixlib.defmatrix.matrix
        TxN matrix of orthogonality conditions.
    lags: integer
        Number of lags.
    newey_west: boolean, default False
        If False, return equally-weighted S matrix (Hansen-Hodrick).
        If True, return Newey-West matrix.
    demean: boolean, default True
        If True, substract off means in S.
    """
    weight = 1 if newey_west else 0

    T = u.shape[0]

    S = u.T @ u / T

    for k in range(1, lags+1):
        sadd1 = u[k:, :] - u[k:, :].mean(axis=0) if demean else u[k:, :]
        sadd2 = u[:-k, :] - u[:-k, :].mean(axis=0) if demean else u[:-k, :]

        sadd = sadd1.T @ sadd2 / T
        sadd = sadd + sadd.T
        S += (1-weight*k/(lags + 1)) * sadd

    return S
